@@ -96,8 +96,13 @@ edit_session_name (gchar *title, gchar **editsession,
   GtkWidget *vbox;
   GtkWidget *entry;
   GtkWidget *frame;
-  *dialog = gnome_dialog_new (title, GNOME_STOCK_BUTTON_OK,
-				GNOME_STOCK_BUTTON_CANCEL,
+  char      *old_session_name = NULL;
+
+  if (*editsession)
+    old_session_name = g_strdup (*editsession);
+
+  *dialog = gnome_dialog_new (title, GNOME_STOCK_BUTTON_CANCEL,
+				GNOME_STOCK_BUTTON_OK,
 				NULL);
   gnome_dialog_close_hides (GNOME_DIALOG (*dialog), TRUE);
   gtk_window_set_policy (GTK_WINDOW (*dialog), FALSE, TRUE, FALSE);
@@ -119,8 +124,13 @@ edit_session_name (gchar *title, gchar **editsession,
 
   gtk_widget_show_all (*dialog); 
 
-  while (gnome_dialog_run (GNOME_DIALOG (*dialog)) == 0) {  
+  while (gnome_dialog_run (GNOME_DIALOG (*dialog)) == 1) {  
+    gboolean name_changed = TRUE;
+
     *editsession = gtk_editable_get_chars (GTK_EDITABLE (entry),0,-1);
+    if (old_session_name && !strcmp (old_session_name, *editsession))
+      name_changed = FALSE;
+
     if (is_blank (*editsession)) {
       GtkWidget *msgbox;
       gtk_widget_show (*dialog);
@@ -131,7 +141,7 @@ edit_session_name (gchar *title, gchar **editsession,
       gnome_dialog_set_parent (GNOME_DIALOG (msgbox), GTK_WINDOW (*dialog));
       gnome_dialog_run (GNOME_DIALOG (msgbox));
     } 
-    else if(checkduplication(*editsession, sess_list)) {
+    else if (name_changed && checkduplication (*editsession, sess_list)) {
       GtkWidget *msgbox;
       gtk_widget_show (*dialog);
       msgbox = gnome_message_box_new (_("The session name already exists"),
@@ -144,12 +154,15 @@ edit_session_name (gchar *title, gchar **editsession,
     else {
       gtk_widget_destroy (*dialog);
       *dialog = NULL;
+      g_free (old_session_name);
       return TRUE;
     }
   }
   
   gtk_widget_destroy (*dialog);
   *dialog = NULL;
+  g_free (old_session_name);
+
   return FALSE;
 }
 
@@ -170,6 +183,7 @@ session_list_add_dialog (GSList **sess_list, GtkWidget **dialog)
   if(edit_session_name (_("Add a new session"), &new_session_name, sess_list, dialog)) {
     *sess_list = g_slist_prepend (*sess_list,g_strdup(new_session_name));
     *sess_list = g_slist_sort (*sess_list, (GCompareFunc)session_compare);
+    spc_write_state ();
   }
   g_free(new_session_name);
 }
@@ -186,6 +200,7 @@ session_list_delete (GSList **sess_list, const gchar *old_session_name,
     }
   }
   *sess_list = g_slist_sort (*sess_list, (GCompareFunc)session_compare);
+  spc_write_state ();
 }
 
 void
@@ -206,6 +221,7 @@ session_list_edit_dialog (GSList **sess_list, const gchar *old_session_name,
     *sess_list = g_slist_sort (*sess_list, (GCompareFunc)session_compare);
     g_hash_table_insert(*hash, g_strdup(edited_session_name), g_strdup(old_session_name));
     g_hash_table_insert(*hash, g_strdup(old_session_name), g_strdup(edited_session_name)); 
+   spc_write_state ();
   }
 }
 
