@@ -34,6 +34,11 @@ Author:  Ralph Mor, X Consortium
 #include <X11/Xmu/WinUtil.h>
 #include <libgnome/libgnome.h>
 #include <unistd.h>
+#include <netdb.h>
+
+#if !defined (NI_MAXHOST)
+#define NI_MAXHOST MAXHOSTNAMELEN
+#endif
 
 XtAppContext appContext;
 Display *disp;
@@ -44,6 +49,7 @@ Atom wmStateAtom;
 Atom smClientIdAtom;
 Atom wmClientLeaderAtom;
 Atom dtSaveModeAtom;
+Atom dtRestoreModeAtom;
 
 Bool debug = 0;
 
@@ -199,10 +205,10 @@ int *newstring;
     }
     else
     {
-	char hostnamebuf[80];
+	char hostnamebuf [NI_MAXHOST];
 	char *firstDot;
 
-	gethostname (hostnamebuf, sizeof hostnamebuf);
+	gethostname (hostnamebuf, NI_MAXHOST);
 	firstDot = strchr (hostnamebuf, '.');
 
 	if (!firstDot)
@@ -216,7 +222,7 @@ int *newstring;
 	    char *newptr;
 
 	    newptr = (char *) malloc (bytes);
-	    sprintf (newptr, "%s.%s", name, firstDot + 1);
+	    snprintf (newptr, NI_MAXHOST, "%s.%s", name, firstDot + 1);
 
 	    *newstring = 1;
 	    return (newptr);
@@ -238,7 +244,7 @@ Bool has_WM_SAVEYOURSELF;
 
     if (!winInfo->got_first_save_yourself)
     {
-	char userId[20], restartService[80];
+	char userId[20], restartService [NI_MAXHOST + 10];
 	char *fullyQuantifiedName;
 	int newstring;
 
@@ -259,7 +265,7 @@ Bool has_WM_SAVEYOURSELF;
     
 	fullyQuantifiedName = CheckFullyQuantifiedName (
 	    (char *) winInfo->wm_client_machine.value, &newstring);
-	sprintf (restartService, "rstart-rsh/%s", fullyQuantifiedName);
+	snprintf (restartService, NI_MAXHOST + 10, "rstart-rsh/%s", fullyQuantifiedName);
 	if (newstring)
 	    free (fullyQuantifiedName);
 
@@ -633,7 +639,8 @@ Window window;
     newptr->wm_command_count = 0;
     newptr->class.res_name = NULL;
     newptr->class.res_class = NULL;
-    newptr->wm_name = NULL;
+    newptr->wm_name.value = NULL;
+    newptr->wm_name.nitems = 0;
     newptr->wm_client_machine.value = NULL;
     newptr->wm_client_machine.nitems = 0;
     newptr->has_save_yourself = 0;
@@ -666,8 +673,8 @@ WinInfo *winptr;
 	if (ptr->wm_command)
 	    XFreeStringList (ptr->wm_command);
 	
-	if (ptr->wm_name)
-	    XFree (ptr->wm_name);
+	if (ptr->wm_name.value)
+	    XFree (ptr->wm_name.value);
 	
 	if (ptr->wm_client_machine.value)
 	    XFree (ptr->wm_client_machine.value);
@@ -743,7 +750,7 @@ WinInfo *winptr;
 
     if (!HasXSMPsupport (leader_winptr->window))
     {
-	XFetchName (disp, leader_winptr->window, &leader_winptr->wm_name);
+	XGetWMName (disp, leader_winptr->window, &leader_winptr->wm_name);
 
 	XGetCommand (disp, leader_winptr->window,
 	    &leader_winptr->wm_command,
@@ -754,7 +761,8 @@ WinInfo *winptr;
 	XGetWMClientMachine (disp, leader_winptr->window,
 	    &leader_winptr->wm_client_machine);
 
-	if (leader_winptr->wm_name != NULL &&
+	if (leader_winptr->wm_name.value != NULL &&
+	    leader_winptr->wm_name.nitems != 0 &&
 	    leader_winptr->wm_command != NULL &&
 	    leader_winptr->wm_command_count > 0 &&
 	    leader_winptr->class.res_name != NULL &&
@@ -1430,6 +1438,7 @@ char **argv;
     smClientIdAtom = XInternAtom (disp, "SM_CLIENT_ID", False);
     wmClientLeaderAtom = XInternAtom (disp, "WM_CLIENT_LEADER", False);
     dtSaveModeAtom = XInternAtom (disp, "_DT_SAVE_MODE", False);
+    dtRestoreModeAtom = XInternAtom (disp, "_DT_RESTORE_MODE", False);
 
     /* Workaround for utterly broken CDE applications.
      * See http://bugzilla.gnome.org/show_bug.cgi?id=81343
@@ -1438,6 +1447,9 @@ char **argv;
     XChangeProperty (disp, RootWindow (disp, 0),
                      dtSaveModeAtom, XA_STRING, 8, PropModeReplace,
 		     "gnome", strlen ("gnome") + 1);
+    XChangeProperty (disp, RootWindow (disp, 0),
+                     dtRestoreModeAtom, XA_STRING, 8, PropModeReplace,
+		     "gnome", strlen ("gnome") + 1);
 
     for (i = 0; i < ScreenCount (disp); i++)
     {
